@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from threading import Thread
-import time
+import logging
 
 from tornado.ioloop import IOLoop
 from tornado.httpclient import AsyncHTTPClient, HTTPRequest
@@ -19,8 +18,8 @@ from octopus.model import Response
 class TornadoOctopus(object):
     def __init__(
             self, concurrency=10, auto_start=False, cache=False,
-            expiration_in_seconds=30, request_timeout_in_seconds=60,
-            connect_timeout_in_seconds=30, ignore_pycurl=False):
+            expiration_in_seconds=30, request_timeout_in_seconds=10,
+            connect_timeout_in_seconds=5, ignore_pycurl=False):
 
         self.concurrency = concurrency
         self.auto_start = auto_start
@@ -36,34 +35,20 @@ class TornadoOctopus(object):
         self.url_queue = []
 
         if PYCURL_AVAILABLE and not self.ignore_pycurl:
+            logging.debug('pycurl is available, thus Octopus will be using it instead of tornado\'s simple http client.')
             AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")
-        #else:
-            #logging.warn(
-                #'pycurl is not enabled and performance will suffer badly, '
-                #'as well as being less reliable. Please consider installing '
-                #'the latest versions of libcurl an pycurl.'
-            #)
-
-        self._stopped = True
 
         if auto_start:
+            logging.debug('Auto starting...')
             self.start()
 
     def start(self):
+        logging.debug('Creating IOLoop and http_client.')
         self.ioloop = IOLoop()
         self.http_client = AsyncHTTPClient(io_loop=self.ioloop)
-        #self.start_alternate_thread()
 
-    #def start_alternate_thread(self):
-        #print "starting ioloop"
-        #self.alternate_thread = Thread(target=self.start_ioloop, args=(self.ioloop, ))
-        #self.alternate_thread.daemon = True
-        #self.alternate_thread.start()
-
-    def start_ioloop(self, ioloop):
-        ioloop.start()
-
-    def from_tornado_response(self, url, response):
+    @classmethod
+    def from_tornado_response(cls, url, response):
         cookies = response.request.headers.get('Cookie', '')
         if cookies:
             cookies = dict([cookie.split('=') for cookie in cookies.split(';')])
@@ -77,40 +62,27 @@ class TornadoOctopus(object):
             request_time=response.request_time
         )
 
-    def handle_request(self, url, callback):
-        def handle(response):
-            response = self.from_tornado_response(url, response)
-
-            if self.cache:
-                self.response_cache.put(url, response)
-
-            self.running_urls -= 1
-            callback(url, response)
-
-            if self.running_urls < self.concurrency and self.url_queue:
-                request_url, handler, method, kw = self.url_queue.pop()
-                self.fetch(request_url, handler, method, **kw)
-
-            if self.running_urls < 1:
-                self.stop()
-
-        return handle
-
     def enqueue(self, url, handler, method='GET', **kw):
+        logging.debug('Enqueueing %s...' % url)
+
         if self.cache:
             response = self.response_cache.get(url)
 
             if response is not None:
+                logging.debug('Cache hit on %s.' % url)
                 handler(url, response)
                 return
 
         if self.running_urls < self.concurrency:
+            logging.debug('Queue has space available for fetching %s.' % url)
             self.fetch(url, handler, method, **kw)
         else:
+            logging.debug('Queue is full. Enqueueing %s for future fetch.' % url)
             self.url_queue.append((url, handler, method, kw))
 
     def fetch(self, url, handler, method, **kw):
         self.running_urls += 1
+        logging.info('Fetching %s...' % url)
 
         request = HTTPRequest(
             url=url,
@@ -122,21 +94,46 @@ class TornadoOctopus(object):
 
         self.http_client.fetch(request, self.handle_request(url, handler))
 
-        #if self._stopped:
-            #self.start_alternate_thread()
+    def handle_request(self, url, callback):
+        def handle(response):
+            response = self.from_tornado_response(url, response)
+            logging.info('Got response(%s) from %s.' % (response.status_code, url))
+
+            if self.cache:
+                logging.debug('Putting %s into cache.' % url)
+                self.response_cache.put(url, response)
+
+            self.running_urls -= 1
+            callback(url, response)
+
+            if self.running_urls < self.concurrency and self.url_queue:
+                request_url, handler, method, kw = self.url_queue.pop()
+                logging.debug('Queue has space available for fetching %s.' % request_url)
+                self.fetch(request_url, handler, method, **kw)
+
+            if self.running_urls < 1:
+                logging.debug('Nothing else to get. Stopping Octopus...')
+                self.stop()
+
+        return handle
 
     def handle_wait_timeout(self, signal_number, frames):
-        print "HELLO FROM SIGNAL %s" % signal_number
+        logging.debug('Timeout waiting for IOLoop to finish. Stopping IOLoop manually.')
         self.ioloop.stop()
 
     def wait(self, timeout=10):
         if not self.url_queue and not self.running_urls:
+            logging.debug('No urls to wait for. Returning immediately.')
             return
 
-        self.ioloop.set_blocking_signal_threshold(timeout, self.handle_wait_timeout)
+        if timeout:
+            logging.debug('Waiting for urls to be retrieved for %s seconds.' % timeout)
+            self.ioloop.set_blocking_signal_threshold(timeout, self.handle_wait_timeout)
+        else:
+            logging.debug('Waiting for urls to be retrieved.')
+
         self.ioloop.start()
 
     def stop(self):
-        print "stopping ioloop"
+        logging.debug('Stopping IOLoop...')
         self.ioloop.stop()
-        self._stopped = True
